@@ -955,26 +955,38 @@ test_histogramDP <- function(){
   if (!is.character(a)) print("FAIL");
   print("")
 
+  print("         Bad lower.bound:")
+  a = tryCatch(histogramDP(data1d,1,c(1,2),3),
+               error=function(e) print(paste("PASS --",e)));
+  if (!is.character(a)) print("FAIL");
+  print("")
+
+  print("         Bad upper.bound:")
+  a = tryCatch(histogramDP(data1d,1,-3,c(1,2)),
+               error=function(e) print(paste("PASS --",e)));
+  if (!is.character(a)) print("FAIL");
+  print("")
+
   print("         Bad mechanism:")
-  a = tryCatch(histogramDP(data1d,1,mechanism='abc',type.DP='pDP'),
+  a = tryCatch(histogramDP(data1d,1,-3,3,mechanism='abc',type.DP='pDP'),
                error=function(e) print(paste("PASS --",e)));
   if (!is.character(a)) print("FAIL");
   print("")
 
   print("         Mechanism Gaussian with delta=0:")
-  a = tryCatch(histogramDP(data1d,1,mechanism='Gaussian',type.DP='pDP'),
+  a = tryCatch(histogramDP(data1d,1,-3,3,mechanism='Gaussian',type.DP='pDP'),
                error=function(e) print(paste("PASS --",e)));
   if (!is.character(a)) print("FAIL");
   print("")
 
   print("         Mechanism analytic with delta=0:")
-  a = tryCatch(histogramDP(data1d,1,mechanism='analytic'),
+  a = tryCatch(histogramDP(data1d,1,-3,3,mechanism='analytic'),
                error=function(e) print(paste("PASS --",e)));
   if (!is.character(a)) print("FAIL");
   print("")
 
   print("         Bad which.sensitivity:")
-  a = tryCatch(histogramDP(data1d,1,which.sensitivity='abc',type.DP='pDP'),
+  a = tryCatch(histogramDP(data1d,1,-3,3,which.sensitivity='abc',type.DP='pDP'),
                error=function(e) print(paste("PASS --",e)));
   if (!is.character(a)) print("FAIL");
   print("")
@@ -984,6 +996,8 @@ test_histogramDP <- function(){
   print("         Simple bounded Laplace:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- 1;
+  lb <- -3;
+  ub <- 9;
   normal <- FALSE;
   ws = 'bounded'
   mech = 'Laplace'
@@ -991,7 +1005,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Simple bounded Laplace");
@@ -1003,6 +1018,8 @@ test_histogramDP <- function(){
   print("         Simple unbounded Laplace:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- 1;
+  lb <- -3;
+  ub <- 9;
   normal <- FALSE;
   ws = 'unbounded'
   mech = 'Laplace'
@@ -1010,7 +1027,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Simple unbounded Laplace");
@@ -1022,6 +1040,8 @@ test_histogramDP <- function(){
   print("         Gaussian pDP:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- 1;
+  lb <- -3;
+  ub <- 9;
   normal <- FALSE;
   ws = 'bounded'
   mech = 'Gaussian'
@@ -1029,7 +1049,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Gaussian pDP");
@@ -1041,6 +1062,8 @@ test_histogramDP <- function(){
   print("         Gaussian aDP:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- .9;
+  lb <- -3;
+  ub <- 9;
   normal <- FALSE;
   ws = 'bounded'
   mech = 'Gaussian'
@@ -1048,7 +1071,8 @@ test_histogramDP <- function(){
   tdp = 'aDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Gaussian aDP");
@@ -1060,6 +1084,8 @@ test_histogramDP <- function(){
   print("         Analytic Gaussian:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- .9;
+  lb <- -3;
+  ub <- 9;
   normal <- FALSE;
   ws = 'bounded'
   mech = 'analytic'
@@ -1067,7 +1093,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Analytic Gaussian");
@@ -1079,6 +1106,8 @@ test_histogramDP <- function(){
   print("         Normalized:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- 1;
+  lb <- -3;
+  ub <- 9;
   normal <- TRUE;
   ws = 'bounded'
   mech = 'Laplace'
@@ -1086,7 +1115,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = FALSE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Normalized");
@@ -1098,6 +1128,8 @@ test_histogramDP <- function(){
   print("         Allow Negative:")
   x <- stats::rnorm(100,mean=3,sd=2);
   eps <- 1;
+  lb <- -3;
+  ub <- 9;
   normal <- TRUE;
   ws = 'bounded'
   mech = 'Laplace'
@@ -1105,7 +1137,8 @@ test_histogramDP <- function(){
   tdp = 'pDP'
   an = TRUE;
 
-  result <- histogramDP(x, eps, "Sturges", normal, ws, mech, delta, tdp, an)
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
 
   # graphics::par(mfrow=c(1,2))
   plot(result,main="Allow Negative");
@@ -1114,6 +1147,28 @@ test_histogramDP <- function(){
   Sys.sleep(2)
   # graphics::par(mfrow=c(1,1))
 
+  print("         Different Bounds:")
+  x <- stats::rnorm(100,mean=3,sd=2);
+  eps <- 1;
+  lb <- 0.2;
+  ub <- 15;
+  normal <- FALSE;
+  ws = 'bounded'
+  mech = 'Laplace'
+  delta = 0
+  tdp = 'pDP'
+  an = FALSE;
+
+  result <- histogramDP(x, eps, lb, ub, "Sturges", normal, ws,
+                        mech, delta, tdp, an)
+
+  # graphics::par(mfrow=c(1,2))
+  plot(result,main="Different Bounds");
+  graphics::hist(x,freq=FALSE,main="Original");
+  print("Verify similarity...")
+  Sys.sleep(2)
+  # graphics::par(mfrow=c(1,1))
+
   ### END TEST FUNCTIONALITY ###
 }
 
